<?php

namespace apexl\Io\modules\component\entities;

use apexl\Io\includes\Entity;

class componentTypeEntity extends Entity
{
    public function __construct()
    {
        parent::__construct('component_types', 'type');
        $this->setRequiredData([
            'name',
            'type',
            'namespace',
            'module',
        ]);
    }
}
