<?php

namespace apexl\Io\modules\component;

use apexl\Io\includes\Module;
use apexl\Io\includes\System;
use apexl\Io\modules\component\services\Component;

/**
 * Base module file, provides required methods to register things like routes and middleware.
 * Class installModule
 * @package apexl\Io\modules\install
 */
class componentModule extends Module
{
    public function __construct()
    {
        parent::__construct();
    }

    public function routes(): void
    {
    }

    public function schema(): object
    {
        return System::loadSchema(__DIR__.'/schema/schema.json');
    }
}
