<?php

namespace apexl\Io\modules\component\controllers;

use apexl\Io\services\Component;
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class componentController
{
    protected $componentGroup;
    protected $component;
    public function __construct(Component $component)
    {
        $this->component = $component;
    }

    public function getComponent(Request $request, Response $response, $args)
    {
    }
}
